@@ -329,11 +329,11 @@ let openFileOut' fspath path kind len =
       let fullpath = Fspath.concat fspath path in
       let flags = [Unix.O_WRONLY; Unix.O_CREAT; Unix.O_CLOEXEC] in
       let perm = if Prefs.read Props.dontChmod then Props.perms Props.fileDefault else 0o600 in
-      begin match Util.osType with
-        `Win32 ->
+      begin match Sys.win32 with
+      | true ->
           Fs.open_out_gen
             [Open_wronly; Open_creat; Open_excl; Open_binary] perm fullpath
-      | `Unix ->
+      | false ->
           let fd =
             try
               Fs.openfile fullpath (Unix.O_EXCL :: flags) perm
@@ -878,6 +878,7 @@ let rec registerFileTransfer pathTo fp f =
 let copyprog =
   Prefs.createString "copyprog" "rsync --partial --inplace --compress"
     ~category:(`Advanced `General)
+    ~deprecated:true
     "external program for copying large files"
     ("A string giving the name of an "
      ^ "external program that can be used to copy large files efficiently  "
@@ -889,6 +890,7 @@ let copyprogrest =
   Prefs.createString
     "copyprogrest" "rsync --partial --append-verify --compress"
     ~category:(`Advanced `General)
+    ~deprecated:true
     "variant of copyprog for resuming partial transfers"
     ("A variant of {\\tt copyprog} that names an external program "
      ^ "that should be used to continue the transfer of a large file "
@@ -901,6 +903,7 @@ let copyprogrest =
 let copythreshold =
   Prefs.createInt "copythreshold" (-1)
     ~category:(`Advanced `General)
+    ~deprecated:true
     "use copyprog on files bigger than this (if >=0, in Kb)"
     ("A number indicating above what filesize (in kilobytes) Unison should "
      ^ "use the external "
@@ -917,6 +920,7 @@ let () = Prefs.markRemoved "copyquoterem"
 let copymax =
   Prefs.createInt "copymax" 1
     ~category:(`Advanced `General)
+    ~deprecated:true
     "maximum number of simultaneous copyprog transfers"
     ("A number indicating how many instances of the external copying utility \
       Unison is allowed to run simultaneously (default to 1).")
@@ -931,6 +935,7 @@ let formatConnectionInfo root =
            (Globals.parsedClRawRoots ())
       with
         Clroot.ConnectByShell (_,rawhost,uo,_,_) ->
+          let rawhost = if String.contains rawhost ':' then "[" ^ rawhost ^ "]" else rawhost in
             (match uo with None -> "" | Some u -> u ^ "@")
           ^ rawhost ^ ":"
           (* Note that we don't do anything with the port -- hopefully
