@@ -117,7 +117,7 @@ let rec safe_waitpid pid =
         Unix.sleepf 0.002;
         let dt = Unix.gettimeofday () -. t in
         if dt >= 0.5 && st = 0 then begin
-          kill_noerr Sys.(if os_type = "Win32" then sigkill else sigterm);
+          kill_noerr Sys.(if win32 then sigkill else sigterm);
           aux 1
         end else if dt >= 2.0 && st = 1 then begin
           kill_noerr Sys.sigkill;
@@ -132,9 +132,8 @@ let rec safe_waitpid pid =
 let term_sessions = Hashtbl.create 3
 
 external win_create_process_pty :
-  string -> string -> pty ->
-  Unix.file_descr -> Unix.file_descr -> Unix.file_descr -> int
-  = "w_create_process_pty" "w_create_process_pty_native"
+  string -> string -> pty -> Unix.file_descr -> Unix.file_descr -> int =
+  "w_create_process_pty"
 
 let make_cmdline args =
   let maybe_quote f =
@@ -143,8 +142,8 @@ let make_cmdline args =
     else f in
   String.concat " " (List.map maybe_quote (Array.to_list args))
 
-let create_process_pty prog args pty fd1 fd2 fd3 =
-  win_create_process_pty prog (make_cmdline args) pty fd1 fd2 fd3
+let create_process_pty prog args pty fd1 fd2 =
+  win_create_process_pty prog (make_cmdline args) pty fd1 fd2
 
 let protect f g =
   try f () with Sys_error _ | Unix.Unix_error _ as e ->
@@ -159,7 +158,7 @@ let finally f g =
 external win_alloc_console : unit -> Unix.file_descr option = "win_alloc_console"
 
 let fallback_session cmd args new_stdin new_stdout new_stderr =
-  if Sys.os_type = "Win32" then begin
+  if Sys.win32 then begin
     (* OCaml's [Unix.create_process] hides the Windows console window of
        the child process unless the parent process already has a console.
        This is unsuitable for running interactive child processes like
@@ -195,7 +194,8 @@ let win_create_session cmd args new_stdin new_stdout new_stderr =
   | Some ((masterIn, masterOut), pty, (conIn, conOut)) ->
       safe_close conIn;
       let create_proc () =
-        create_process_pty cmd args pty new_stdin new_stdout conOut in
+        (* Child's stderr is always connected to pty (conOut, effectively). *)
+        create_process_pty cmd args pty new_stdin new_stdout in
       let childPid =
         protect (fun () -> finally create_proc
                                    (fun () -> safe_close conOut))
@@ -233,7 +233,9 @@ let unix_create_session cmd args new_stdin new_stdout new_stderr =
             (* new_stderr will be used by parent process only. *)
             if new_stderr <> Unix.stderr then safe_close new_stderr;
             Unix.close masterFd;
-            ignore (Unix.setsid ());
+            (* [Unix.setsid] is not implemented on Cygwin, reason unknown.
+               It will be called by [setControllingTerminal] instead. *)
+            if not Sys.cygwin then ignore (Unix.setsid ());
             setControllingTerminal slaveFd;
             (* WARNING: SETTING ECHO TO FALSE! *)
             let tio = Unix.tcgetattr slaveFd in
@@ -334,7 +336,7 @@ let parseCtrlSeq s =
         begin
           match ch with
           | '\024' | '\026' -> state No (* CAN, SUB *)
-          | '\000'..'\025' -> add_char ch (* Control charaters (roughly) *)
+          | '\000'..'\025' -> add_char ch (* Control characters (roughly) *)
           | '\027' -> state Escape
           | '\048'..'\126' -> state No (* Final *)
           | '\127'..'\255' -> state No (* Invalid *)
@@ -344,7 +346,7 @@ let parseCtrlSeq s =
         begin
           match ch with
           | '\024' | '\026' -> state No (* CAN, SUB *)
-          | '\000'..'\025' -> add_char ch (* Control charaters (roughly) *)
+          | '\000'..'\025' -> add_char ch (* Control characters (roughly) *)
           | '\027' -> state Escape
           | '\064'..'\126' -> (* Final *)
               begin
@@ -365,7 +367,7 @@ let parseCtrlSeq s =
           match ch with
           | '\024' | '\026' -> state No (* CAN, SUB *)
           | '\007' -> state No (* BEL *)
-          | '\000'..'\025' -> add_char ch (* Control charaters (roughly) *)
+          | '\000'..'\025' -> add_char ch (* Control characters (roughly) *)
           | '\027' -> state OSCEsc
           | _ -> ()
         end
@@ -379,7 +381,7 @@ let parseCtrlSeq s =
         begin
           match ch with
           | '\024' | '\026' -> state No (* CAN, SUB *)
-          | '\000'..'\025' -> add_char ch (* Control charaters (roughly) *)
+          | '\000'..'\025' -> add_char ch (* Control characters (roughly) *)
           | '\027' -> state StringEsc
           | _ -> ()
         end
@@ -424,7 +426,7 @@ type termInteract = {
   endInput : unit -> unit }
 
 (* Read messages from the terminal and use the callback to get an answer *)
-let handlePasswordRequests (fdIn, fdOut) {userInput; endInput} isReady =
+let handlePasswordRequests (fdIn, fdOut) {userInput; endInput} =
   let scrollback = Buffer.create 32 in
   let extract () =
     let s = Buffer.contents scrollback in
@@ -445,7 +447,7 @@ let handlePasswordRequests (fdIn, fdOut) {userInput; endInput} isReady =
   let sendResponse s =
     Lwt.catch
       (fun () ->
-        if isReady () || !ended then Lwt.return 0
+        if !ended then Lwt.return 0
         else Lwt_unix.write_substring fdOut (s ^ "\n") 0 (String.length s + 1))
       (terminalError "writing to shell terminal")
   in
@@ -469,30 +471,30 @@ let handlePasswordRequests (fdIn, fdOut) {userInput; endInput} isReady =
        remote shell process dies. The reading will end (return 0 or an error)
        when the pty is closed.
        The only way to stop the reading loop without closing the pty is to
-       signal [isReady]. *)
+       signal [connectionReady] or [closeInput]. *)
     Lwt.catch
       (fun () -> Lwt_unix.read fdIn buf 0 blen)
-      (fun ex -> if isReady () || !ended then Lwt.return 0 else Lwt.fail ex)
+      (fun ex -> if !ended then Lwt.return 0 else Lwt.fail ex)
     >>= function
     | 0 -> Lwt.return ()
     | len ->
         Buffer.add_string scrollback (Bytes.sub_string buf 0 len);
-        if isReady () then begin (* The shell connection has been established *)
-          closeInput ();
+        if !ended then begin (* The shell connection has been established *)
           Lwt.return ()
         end else begin
           Lwt.ignore_result (Lwt_unix.sleep 0.05 >>= fun () -> (* Give time for connection checks *)
-            Lwt.return (if not !ended && not (isReady ()) then promptUser ()));
+            Lwt.return (if not !ended then promptUser ()));
           loop ()
         end
   in
   let readTerm = Lwt.catch loop (terminalError "reading from shell terminal") in
-  let extractRemainingOutput clean =
+  let handleReqs = readTerm >>= fun () -> Lwt.return (extract ()) in
+  let connectionReady () = closeInput (); extract () in
+  let extractRemainingOutput () =
     closeInput ();
     (* Give a final chance of reading the error output from the ssh process. *)
     let timeout = Lwt_unix.sleep 0.3 in
     Lwt.choose [readTerm; timeout] >>= fun () ->
-    if not clean then Lwt.return (extract ())
-    else Lwt.return (Util.trimWhitespace (processEscapes (extract ())))
+    Lwt.return (Util.trimWhitespace (processEscapes (extract ())))
   in
-  (readTerm, extractRemainingOutput)
+  (connectionReady, handleReqs, extractRemainingOutput)
