@@ -136,7 +136,7 @@ let repeat =
      ^ "beginning again. When the argument is \\verb|watch|, Unison relies on "
      ^ "an external file monitoring process to synchronize whenever a change "
      ^ "happens.  You can combine the two with a \\verb|+| character to use "
-     ^ "file monitoring and also do a full scan every specificed number of "
+     ^ "file monitoring and also do a full scan every specified number of "
      ^ "seconds.  For example, \\verb|watch+3600| will react to changes "
      ^ "immediately and additionally do a full scan every hour.")
     (fun _ -> parseRepeat)
@@ -226,7 +226,7 @@ let replicaContent2string rc sep =
       "absent"
   | _, `Unchanged ->
       "unchanged "
-     ^(Util.truncateString (Fileinfo.type2string rc.typ) 7)
+     ^(Util.padto 7 (Util.truncateString (Fileinfo.type2string rc.typ) 7))
      ^ sep
      ^ replicaContentDesc rc
   | `ABSENT, `Deleted -> "deleted"
@@ -288,7 +288,7 @@ let details2string theRi sep =
   | Different {rc1 = rc1; rc2 = rc2} ->
       let root1str, root2str =
         roots2niceStrings 12 (Globals.roots()) in
-      Printf.sprintf "%s : %s\n%s : %s"
+      Printf.sprintf "%-12s : %s\n%-12s : %s"
         root1str (replicaContent2string rc1 sep)
         root2str (replicaContent2string rc2 sep)
 
@@ -314,7 +314,7 @@ let displayPath previousPath path =
 
 let roots2string () =
   let replica1, replica2 = roots2niceStrings 12 (Globals.roots()) in
-  (Printf.sprintf "%s   %s       " replica1 replica2)
+  (Printf.sprintf "%-12s   %-12s       " replica1 replica2)
 
 type action = AError | ASkip of bool | ALtoR of bool | ARtoL of bool | AMerge
 
@@ -353,10 +353,10 @@ let reconItem2string oldPath theRI status =
 let exn2string e =
   match e with
      Sys.Break      -> "Terminated!"
-   | Util.Fatal(s)  -> Printf.sprintf "Fatal error: %s" s
-   | Util.Transient(s) -> Printf.sprintf "Error: %s" s
+   | Util.Fatal s   -> s
+   | Util.Transient s -> s
    | Unix.Unix_error (err, fun_name, arg) ->
-       Printf.sprintf "Uncaught unix error: %s failed%s: %s%s\n%s"
+       Printf.sprintf "Uncaught unix error (please report a bug): %s failed%s: %s%s\n%s"
          fun_name
          (if String.length arg > 0 then Format.sprintf " on \"%s\"" arg else "")
          (Unix.error_message err)
@@ -364,9 +364,14 @@ let exn2string e =
             Unix.EUNKNOWNERR n -> Format.sprintf " (code %d)" n
           | _                  -> "")
          (Printexc.get_backtrace ())
+   | Stack_overflow ->
+       "Stack overflow. This could indicate a programming error.\n\n\
+         Technical information in case you need to report a bug:\n"
+       ^ (Printexc.get_backtrace ())
    | Invalid_argument s ->
-       Printf.sprintf "Invalid argument: %s\n%s" s (Printexc.get_backtrace ())
-   | other -> Printf.sprintf "Uncaught exception %s\n%s"
+       Printf.sprintf "Invalid argument (please report a bug): %s\n%s"
+         s (Printexc.get_backtrace ())
+   | other -> Printf.sprintf "Uncaught exception (please report a bug): %s\n%s"
        (Printexc.to_string other) (Printexc.get_backtrace ())
 
 (* precondition: uc = File (Updates(_, ..) on both sides *)
@@ -488,17 +493,22 @@ let calcETA rem rate =
   let t = truncate (rem /. rate +. 0.5) in
   (* Estimating the remaining time is not accurate. Reduce the display
      precision (and reduce more when longer time remaining). *)
-  let h, (m, sec) =
-    if t >= 3420 then
-      let u = t + 180 in u / 3600, (((u mod 3600) / 300) * 5, 0)
-    else
-      0,
-      if t >= 2640 then ((t + 180) / 300) * 5, 0
-      else if t >= 1800 then ((t + 119) / 120) * 2, 0
-      else if t >= 120 then let u = t + 15 in u / 60, ((u mod 60) / 30) * 30
-      else t / 60, t mod 60
-  in
-  Printf.sprintf "%02d:%02d:%02d" h m sec
+  if t >= 86220 then
+    let u = t + 180 in
+    Printf.sprintf "%dd %02d:%02d:00" (u / 86400) ((u mod 86400) / 3600)
+      (((u mod 3600) / 300) * 5)
+  else
+    let h, (m, sec) =
+      if t >= 3420 then
+        let u = t + 180 in u / 3600, (((u mod 3600) / 300) * 5, 0)
+      else
+        0,
+        if t >= 2640 then ((t + 180) / 300) * 5, 0
+        else if t >= 1800 then ((t + 119) / 120) * 2, 0
+        else if t >= 120 then let u = t + 15 in u / 60, ((u mod 60) / 30) * 30
+        else t / 60, t mod 60
+    in
+    Printf.sprintf "%02d:%02d:%02d" h m sec
 
 let movAvg curr prev ?(c = 1.) deltaTime avgPeriod =
   if Float.is_nan prev then curr else
@@ -649,12 +659,15 @@ let transportStart () = Transport.logStart ()
 let transportFinish () = Transport.logFinish ()
 
 let transportItems items pRiThisRound makeAction =
+  if Abort.isAll () then () else
   let waiter = Lwt.wait () in
   let outstanding = ref 0 in
   let starting () = incr outstanding in
   let completed () =
     decr outstanding;
-    if !outstanding = 0 then begin
+    if !outstanding = 0 || (!outstanding = 1 && Abort.isAll ()) then begin
+      (* If there is a stop request then we might not get the [completed]
+         notice for the dispense loop itself, so we also stop at count 1. *)
       try Lwt.wakeup waiter () with Invalid_argument _ -> ()
     end
   in
@@ -682,7 +695,10 @@ let transportItems items pRiThisRound makeAction =
   in
   let rec dispenseAction () =
     let i = !idx in
-    if i < im then begin
+    if Abort.isAll () then begin
+      stopDispense ();
+      dispenseDone ()
+    end else if i < im then begin
       let item = items.(i) in
       incr idx;
       if pRiThisRound item then
@@ -773,7 +789,7 @@ let architecture =
     "architecture"
     Umarshal.unit
     Umarshal.(prod3 bool bool bool id id)
-    (fun (_,()) -> return (Util.osType = `Win32, Osx.isMacOSX, Util.isCygwin))
+    (fun (_,()) -> return (Sys.win32, Osx.isMacOSX, Sys.cygwin))
 
 (* During startup the client determines the case sensitivity of each root.
    If any root is case insensitive, all roots must know this -- it's
@@ -786,12 +802,12 @@ let validateAndFixupPrefs () =
     Safelist.exists (fun (isWin, _, _) -> isWin) archs in
   let allHostsAreRunningWindows =
     Safelist.for_all (fun (isWin, _, _) -> isWin) archs in
-  let someHostIsRunningBareWindows =
-    Safelist.exists (fun (isWin, _, isCyg) -> isWin && not isCyg) archs in
+  let someHostIsRunningCygwin =
+    Safelist.exists (fun (_, _, isCyg) -> isCyg) archs in
   let someHostRunningOsX =
     Safelist.exists (fun (_, isOSX, _) -> isOSX) archs in
   let someHostIsCaseInsensitive =
-    someHostIsRunningWindows || someHostRunningOsX in
+    someHostIsRunningWindows || someHostRunningOsX || someHostIsRunningCygwin in
   if Prefs.read Globals.fatFilesystem then begin
     Prefs.overrideDefault Props.permMask 0;
     Prefs.overrideDefault Props.dontChmod true;
@@ -800,9 +816,9 @@ let validateAndFixupPrefs () =
     Prefs.overrideDefault Fileinfo.ignoreInodeNumbers true
   end;
   Case.init someHostIsCaseInsensitive someHostRunningOsX;
-  Props.init someHostIsRunningWindows;
+  Props.init (someHostIsRunningWindows || someHostIsRunningCygwin);
   Osx.init someHostRunningOsX;
-  Fileinfo.init someHostIsRunningBareWindows;
+  Fileinfo.init someHostIsRunningWindows;
   Prefs.set Globals.someHostIsRunningWindows someHostIsRunningWindows;
   Prefs.set Globals.allHostsAreRunningWindows allHostsAreRunningWindows;
   if repeatWatcher () then Prefs.set Fswatch.useWatcher true;
@@ -994,7 +1010,7 @@ let initPrefs ~profileName ~promptForRoots ?(prepDebug = fun () -> ()) () =
   end;
 
   (* Turn on GC messages, if the '-debug gc' flag was provided *)
-  Gc.set {(Gc.get ()) with Gc.verbose = if Trace.enabled "gc" then 0x3F else 0};
+  Gc.set {(Gc.get ()) with Gc.verbose = if Trace.enabled "gc" then 0x43F else 0};
 
   (* Install dummy roots and backup directory if we are running self-tests *)
   if Prefs.read runtests then begin
